package nckd.wnq01.wnq.scm.plugin.form;

import kd.bos.bill.AbstractBillPlugIn;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.dataentity.resource.ResManager;
import kd.bos.dataentity.utils.StringUtils;
import kd.bos.entity.datamodel.IDataModel;
import kd.bos.entity.datamodel.events.ChangeData;
import kd.bos.entity.datamodel.events.PropertyChangedArgs;
import kd.bos.form.IFormView;
import kd.bos.form.ShowType;
import kd.bos.form.cardentry.CardEntry;
import kd.bos.form.control.Control;
import kd.bos.form.control.EntryGrid;
import kd.bos.form.control.Toolbar;
import kd.bos.form.control.events.*;
import kd.bos.form.events.ClosedCallBackEvent;
import kd.bos.form.field.BasedataEdit;
import kd.bos.form.field.events.BeforeF7SelectEvent;
import kd.bos.form.field.events.BeforeF7SelectListener;
import kd.bos.list.ListShowParameter;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;
import kd.bos.servicehelper.BusinessDataServiceHelper;
import kd.fi.er.common.ShowPageUtils;
import kd.fi.er.model.FormModel;
import kd.sdk.plugin.Plugin;

import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @Author: Boil
 * @Date: 2025/5/11 01:02
 * @Version: v1.0.0
 * @Description: 价格批量调整单（新）编辑插件
 **/
public class BatchPriceAdjBaseEditPlugin extends AbstractBillPlugIn implements Plugin, ClickListener, RowClickEventListener, BeforeF7SelectListener {

    @Override
    public void beforeF7Select(BeforeF7SelectEvent beforeF7SelectEvent) {
        String name = beforeF7SelectEvent.getProperty().getName();

        switch (name) {
            case "nckd_xqys_d":
                beforeSelectF7Xqysd(beforeF7SelectEvent);
                break;
            case "nckd_customer":
                beforeSelectF7Customer(beforeF7SelectEvent);
                break;
            case "nckd_saletypes_d":
                beforeSelectF7SaleTypes(beforeF7SelectEvent);
                break;

        }
    }

    private void beforeSelectF7SaleTypes(BeforeF7SelectEvent beforeF7SelectEvent) {
        EntryGrid putentry = this.getView().getControl("nckd_putentry");
        int[] selectRows = putentry.getSelectRows();
        DynamicObject entry = this.getView().getModel().getEntryRowEntity("nckd_putentry", selectRows[0]);
        ListShowParameter formShowParameter = (ListShowParameter) beforeF7SelectEvent.getFormShowParameter();
        List<QFilter> qFilters = new ArrayList<>();
        DynamicObjectCollection nckdSaletypes = entry.getDynamicObjectCollection("nckd_saletypes");
        if (!nckdSaletypes.isEmpty()) {
            List<Object> fbasedataids = nckdSaletypes.stream().map(x -> x.getDynamicObject("fbasedataid").getPkValue()).collect(Collectors.toList());
            qFilters.add(new QFilter("id", QCP.in, fbasedataids));
        }

        if (!qFilters.isEmpty())
            formShowParameter.getListFilterParameter().setQFilters(qFilters);
    }


    private void beforeSelectF7Customer(BeforeF7SelectEvent beforeF7SelectEvent) {
        EntryGrid putentry = this.getView().getControl("nckd_putentry");
        int[] selectRows = putentry.getSelectRows();
        DynamicObject entry = this.getView().getModel().getEntryRowEntity("nckd_putentry", selectRows[0]);
        ListShowParameter formShowParameter = (ListShowParameter) beforeF7SelectEvent.getFormShowParameter();
        List<QFilter> qFilters = new ArrayList<>();
        DynamicObjectCollection nckd_subentryentity = entry.getDynamicObjectCollection("nckd_subentryentity");
        DynamicObject subentry = nckd_subentryentity.get(beforeF7SelectEvent.getRow());
        DynamicObject nckdXqysD = subentry.getDynamicObject("nckd_xqys_d");
        DynamicObject fkNckdSaletypesD = subentry.getDynamicObject("nckd_saletypes_d");
        if (null != nckdXqysD) {
            qFilters.add(new QFilter("nckd_xqy", QCP.equals, nckdXqysD.getPkValue()));
        }

        if (null != fkNckdSaletypesD) {
            qFilters.add(new QFilter("nckd_xslb", QCP.equals, fkNckdSaletypesD.getPkValue()));
        }
        if (!qFilters.isEmpty())
            formShowParameter.getListFilterParameter().setQFilters(qFilters);
    }

    private void beforeSelectF7Xqysd(BeforeF7SelectEvent beforeF7SelectEvent) {
        EntryGrid putentry = this.getView().getControl("nckd_putentry");
        int[] selectRows = putentry.getSelectRows();
        DynamicObject entry = this.getView().getModel().getEntryRowEntity("nckd_putentry", selectRows[0]);
        ListShowParameter formShowParameter = (ListShowParameter) beforeF7SelectEvent.getFormShowParameter();
        List<QFilter> qFilters = new ArrayList<>();
        DynamicObject nckdQuyus = entry.getDynamicObject("nckd_quyus");
        DynamicObjectCollection nckdXqys = entry.getDynamicObjectCollection("nckd_xqys");
        if (null != nckdQuyus) {
            qFilters.add(new QFilter("nckd_qy", QCP.equals, nckdQuyus.getPkValue()));
        }

        if (!nckdXqys.isEmpty()) {
            List<Object> fbasedataids = nckdXqys.stream().map(x -> x.getDynamicObject("fbasedataid").getPkValue()).collect(Collectors.toList());
            qFilters.add(new QFilter("id", QCP.in, fbasedataids));
        }
        if (!qFilters.isEmpty())
            formShowParameter.getListFilterParameter().setQFilters(qFilters);
    }

    @Override
    public void registerListener(EventObject e) {
        super.registerListener(e);

        Control tripEntry = this.getControl("nckd_putentry");
        if (tripEntry instanceof CardEntry) {
            ((CardEntry) tripEntry).addRowClickListener(this);
        }
        Toolbar toolbar = this.getView().getControl("nckd_advcontoolbarap1");
        toolbar.addItemClickListener(this);
        toolbar.addClickListener(this);

        Toolbar toolbar2 = this.getView().getControl("nckd_advcontoolbarap2");
        toolbar2.addItemClickListener(this);
        toolbar2.addClickListener(this);
        this.addClickListeners(new String[]{"nckd_alldai", "nckd_allsan", "nckd_updatetripentry", "nckd_cardentryflexpanel12", "nckd_addtjlabel1", "nckd_addptentry"});
        this.addItemClickListeners("nckd_alldai", "nckd_allsan");

        BasedataEdit nckd_xqys_d = this.getControl("nckd_xqys_d");
        nckd_xqys_d.addBeforeF7SelectListener(this);

        BasedataEdit nckd_saletypes_d = this.getControl("nckd_saletypes_d");
        nckd_saletypes_d.addBeforeF7SelectListener(this);

        BasedataEdit nckd_customer = this.getControl("nckd_customer");
        nckd_customer.addBeforeF7SelectListener(this);
    }


    @Override
    public void beforeItemClick(BeforeItemClickEvent evt) {
        super.beforeItemClick(evt);
        Control source = (Control) evt.getSource();
        if (StringUtils.equals("nckd_alldai", evt.getItemKey())
                || StringUtils.equals("nckd_allsan", evt.getItemKey())
                || StringUtils.equals("nckd_advconbaritemap2", evt.getItemKey())
                || StringUtils.equals("nckd_advconbaritemap3", evt.getItemKey())
                || StringUtils.equals("nckd_advconbaritemap", evt.getItemKey())
                || StringUtils.equals("nckd_advconbaritemap1", evt.getItemKey())) {
            EntryGrid nckdPutentry = this.getView().getControl("nckd_putentry");
            int[] selectRows = nckdPutentry.getSelectRows();
            if (selectRows == null || selectRows.length < 1) {
                this.getView().showErrorNotification("请先选择一个普调分录");
                evt.setCancel(true);
            }
        }
    }

    @Override
    public void beforeClick(BeforeClickEvent evt) {
        super.beforeClick(evt);
    }

    @Override
    public void itemClick(ItemClickEvent evt) {
        super.itemClick(evt);
        String key = evt.getItemKey();
        switch (key) {
            case "nckd_alldai":
                allAddentry("袋装");
                break;
            case "nckd_allsan":
                allAddentry("散装");
                break;
        }
    }

    @Override
    public void click(EventObject evt) {
        super.click(evt);
        String key = ((Control) evt.getSource()).getKey();
        //String entityId = this.getView().getEntityId();
        IDataModel model = this.getModel();
        //DynamicObject dataEntity = model.getDataEntity();
        //String billStatus = dataEntity.getString("billstatus");
        switch (key) {
            case "nckd_updatetripentry":
                int tripEntryIndex = model.getEntryCurrentRowIndex("nckd_putentry");
                this.openPtEntry(tripEntryIndex, true);
                break;
            case "nckd_cardentryflexpanel12":
            case "nckd_addtriplabel1":
            case "nckd_addptentry":
                this.openPtEntry(-1, true);
                break;
            case "nckd_alldai":
                allAddentry("袋装");
                break;
            case "nckd_allsan":
                allAddentry("散装");
                break;
        }
    }

    private void allAddentry(String materialType) {
        DynamicObject[] materialArr = BusinessDataServiceHelper.load("bd_material", "id,masterid,name,nckd_decimalfield,nckd_decimalfield1", new QFilter[]{new QFilter("name", QCP.like, "%" + materialType + "%")});
        if (materialArr.length > 0) {
            EntryGrid nckdPutentry = this.getView().getControl("nckd_putentry");
            int[] selectRows = nckdPutentry.getSelectRows();
            DynamicObjectCollection entity = this.getModel().getEntryEntity("nckd_putentry");
            if (selectRows != null && selectRows.length > 0) {
                DynamicObject entry = entity.get(selectRows[0]);
                DynamicObjectCollection nckdTjdetailentry = entry.getDynamicObjectCollection("nckd_tjdetailentry");
                for (DynamicObject material : materialArr) {
                    DynamicObject subentry = nckdTjdetailentry.addNew();
                    this.getView().getModel().beginInit();
                    subentry.set("nckd_materialhang", material);

                    subentry.set("nckd_cbj", material.getBigDecimal("nckd_decimalfield1"));
                    BigDecimal nckdDecimalfield = null == material.getBigDecimal("nckd_decimalfield") ? BigDecimal.ZERO : material.getBigDecimal("nckd_decimalfield");
                    BigDecimal tiaozjiage = null == entry.getBigDecimal("nckd_tiaozjiage") ? BigDecimal.ZERO : entry.getBigDecimal("nckd_tiaozjiage");
                    String nckdTiaojiatype = entry.getString("nckd_tiaojiatype");
                    BigDecimal newprice = null;
                    if (StringUtils.equals("S", nckdTiaojiatype)) {
                        newprice = nckdDecimalfield.add(tiaozjiage);
                    } else if (StringUtils.equals("X", nckdTiaojiatype)) {
                        newprice = nckdDecimalfield.subtract(tiaozjiage);
                    }
                    subentry.set("nckd_tiaojiafudu", tiaozjiage);
                    subentry.set("nckd_tiaojiatyped", nckdTiaojiatype);
                    subentry.set("nckd_yuanprice", nckdDecimalfield);
                    this.getView().getModel().endInit();
                    subentry.set("nckd_newprice", newprice);
                    this.getView().updateView("nckd_tjdetailentry");
                }

            }
        }
    }

    @Override
    public void propertyChanged(PropertyChangedArgs e) {
        super.propertyChanged(e);
        String name = e.getProperty().getName();
        ChangeData[] changeSet = e.getChangeSet();
        ChangeData changeData = changeSet[0];
        Object newValue = changeData.getNewValue();
        int rowIndex = changeData.getRowIndex();
        int parentRowIndex = changeData.getParentRowIndex();
        IFormView view = this.getView();
        IDataModel model = view.getModel();
        if (StringUtils.equals("nckd_materiel_t", name) || StringUtils.equals("nckd_tiaojiatyped_t", name) || StringUtils.equals("nckd_tiaojiafudu_t", name)) {

            Object nckdTiaojiatypedT = model.getValue("nckd_tiaojiatyped_t", rowIndex);
            Object nckdTiaojiafuduT = model.getValue("nckd_tiaojiafudu_t", rowIndex);
            Object nckdMaterielT = model.getValue("nckd_materiel_t", rowIndex);
            if (nckdTiaojiatypedT != null && nckdTiaojiafuduT != null && nckdMaterielT != null) {
                DynamicObject material = BusinessDataServiceHelper.loadSingleFromCache(((DynamicObject) nckdMaterielT).getPkValue(), ((DynamicObject) nckdMaterielT).getDynamicObjectType().getName());
                model.setValue("nckd_cbj_t", material.getBigDecimal("nckd_decimalfield1"), rowIndex);
                BigDecimal nckdDecimalfield = null == material.getBigDecimal("nckd_decimalfield") ? BigDecimal.ZERO : material.getBigDecimal("nckd_decimalfield");
                BigDecimal tiaozjiage = (BigDecimal) nckdTiaojiafuduT;
                String nckdTiaojiatype = (String) nckdTiaojiatypedT;
                BigDecimal newprice = null;
                if (StringUtils.equals("S", nckdTiaojiatype)) {
                    newprice = nckdDecimalfield.add(tiaozjiage);
                } else if (StringUtils.equals("X", nckdTiaojiatype)) {
                    newprice = nckdDecimalfield.subtract(tiaozjiage);
                }
                view.getModel().beginInit();
                model.setValue("nckd_tiaojiafudu_t", tiaozjiage, rowIndex);
                model.setValue("nckd_tiaojiatyped_t", nckdTiaojiatype, rowIndex);
                model.setValue("nckd_yuanprice_t", nckdDecimalfield, rowIndex);
                view.getModel().endInit();
                model.setValue("nckd_newprice_t", newprice, rowIndex);
                view.updateView("entryentity");

            }
        }
        if (StringUtils.equals("nckd_materialhang", name) && null != newValue) {
            DynamicObjectCollection entity = this.getModel().getEntryEntity("nckd_putentry");
            DynamicObject entry = entity.get(parentRowIndex);
            DynamicObjectCollection nckdTjdetailentry = entry.getDynamicObjectCollection("nckd_tjdetailentry");
            DynamicObject material = BusinessDataServiceHelper.loadSingleFromCache(((DynamicObject) newValue).getPkValue(), ((DynamicObject) newValue).getDynamicObjectType().getName());
            DynamicObject subentry = nckdTjdetailentry.get(rowIndex);

            this.getView().getModel().beginInit();
            subentry.set("nckd_cbj", material.getBigDecimal("nckd_decimalfield1"));
            BigDecimal nckdDecimalfield = null == material.getBigDecimal("nckd_decimalfield") ? BigDecimal.ZERO : material.getBigDecimal("nckd_decimalfield");
            BigDecimal tiaozjiage = null == entry.getBigDecimal("nckd_tiaozjiage") ? BigDecimal.ZERO : entry.getBigDecimal("nckd_tiaozjiage");
            String nckdTiaojiatype = entry.getString("nckd_tiaojiatype");
            BigDecimal newprice = null;
            if (StringUtils.equals("S", nckdTiaojiatype)) {
                newprice = nckdDecimalfield.add(tiaozjiage);
            } else if (StringUtils.equals("X", nckdTiaojiatype)) {
                newprice = nckdDecimalfield.subtract(tiaozjiage);
            }
            subentry.set("nckd_tiaojiafudu", tiaozjiage);
            subentry.set("nckd_tiaojiatyped", nckdTiaojiatype);
            subentry.set("nckd_yuanprice", nckdDecimalfield);
            this.getView().getModel().endInit();
            subentry.set("nckd_newprice", newprice);
            this.getView().updateView("nckd_tjdetailentry");

    }
}



    @Override
    public void closedCallBack(ClosedCallBackEvent closedCallBackEvent) {
        super.closedCallBack(closedCallBackEvent);
    }


    public void openPtEntry(int tripEntryRowIndex, boolean isEnable) {
        IDataModel model = this.getModel();
        String formName = model.getDataEntityType().getName();
        Map<String, Object> customParam = new HashMap();
        customParam.put("tripEntryRowIndex", tripEntryRowIndex);
        customParam.put("formId", formName);
        customParam.put("isEnable", isEnable);
        FormModel formModel = new FormModel(this.getPtEntryEditForm(), ResManager.loadKDString("普调信息", "BatchPriceAdjBaseEdit_0", "scm-sm-formplugin", new Object[0]), "1", true, customParam);
        formModel.setShowType(ShowType.Modal);

        ShowPageUtils.showPage(formModel, this);

    }

    protected String getPtEntryEditForm() {
        return "nckd_ptentryedit";
    }
}
