package nckd.wnq01.wnq.scm.plugin.operate;

import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.dataentity.utils.StringUtils;
import kd.bos.entity.EntityMetadataCache;
import kd.bos.entity.plugin.AbstractOperationServicePlugIn;
import kd.bos.entity.plugin.PreparePropertysEventArgs;
import kd.bos.entity.plugin.args.AfterOperationArgs;
import kd.bos.entity.plugin.args.BeforeOperationArgs;
import kd.bos.entity.plugin.args.BeginOperationTransactionArgs;
import kd.bos.entity.plugin.args.EndOperationTransactionArgs;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;
import kd.bos.servicehelper.BusinessDataServiceHelper;
import kd.sdk.plugin.Plugin;

import java.math.BigDecimal;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 价格批量调整单插件
 */
public class BatchPriceAdjOpPlugin extends AbstractOperationServicePlugIn implements Plugin {
    private static final SimpleDateFormat sdf = new SimpleDateFormat("yyyy年MM月dd日 HH时mm分ss秒");
    @Override
    public void onPreparePropertys(PreparePropertysEventArgs e) {
        super.onPreparePropertys(e);
        List<String> fieldKeys = e.getFieldKeys();
        Set<String> ProjectAllFields = EntityMetadataCache.getDataEntityType(this.billEntityType.getName()).getAllFields().keySet();
        fieldKeys.addAll(ProjectAllFields);
        fieldKeys.add("nckd_tjdetailentry.nckd_materialhang");

    }


    @Override
    public void beginOperationTransaction(BeginOperationTransactionArgs e) {
        super.beginOperationTransaction(e);
        String operationKey = e.getOperationKey();
        if (StringUtils.equals("submit",operationKey) || StringUtils.equals("save",operationKey)){
             DynamicObject[] materialArr = BusinessDataServiceHelper.load("bd_material", "id,masterid,name,nckd_decimalfield,nckd_decimalfield1", new QFilter[]{new QFilter("name", QCP.like, "%袋装%").or(new QFilter("name", QCP.like, "%散装%"))});
            List<DynamicObject> dai = Arrays.stream(materialArr).filter(x -> x.getString("name").contains("袋装")).collect(Collectors.toList());
            List<DynamicObject> san= Arrays.stream(materialArr).filter(x -> x.getString("name").contains("散装")).collect(Collectors.toList());
            for (DynamicObject bill : e.getDataEntities()) {
                joinPtDesc(bill, dai, san);
                joinTstjDesc(bill);
            }

        }
    }

    private void joinTstjDesc(DynamicObject bill) {
        DynamicObjectCollection nckdTstjentry = bill.getDynamicObjectCollection("entryentity");
        for (DynamicObject entry : nckdTstjentry) {
            StringBuilder desc= new StringBuilder();

            DynamicObject nckdQuyus = entry.getDynamicObject("nckd_quyus_t");
            if(null != nckdQuyus){
                desc.append(nckdQuyus.getString("name")).append("区域");
            }
            DynamicObject nckdXqysT = entry.getDynamicObject("nckd_xqys_t");
            if(null != nckdXqysT){
                desc.append(nckdXqysT.getString("name")).append("小区域");
            }
            DynamicObject nckdSaletypes = entry.getDynamicObject("nckd_saletypes_t");
            if(null != nckdSaletypes){
                desc.append(nckdSaletypes.getString("name")).append("销售类别");
            }
            DynamicObject nckdTjjdT = entry.getDynamicObject("nckd_tjjd_t");
            if(null != nckdTjjdT){
                desc.append(nckdTjjdT.getString("name")).append("调价基地");
            }

            DynamicObject nckdCustomerT = entry.getDynamicObject("nckd_customer_t");
            if(null != nckdCustomerT){
                desc.append(nckdCustomerT.getString("name")).append("客户");
            }

            DynamicObject nckdMaterielT = entry.getDynamicObject("nckd_materiel_t");
            if(null != nckdMaterielT){
                desc.append("对").append(nckdCustomerT.getString("name")).append("物料");
            }

            String nckdTiaojiatype = entry.getString("nckd_tiaojiatyped_t");
            if (StringUtils.equals("S",nckdTiaojiatype)){
                desc.append("价格调增");
            } else if (StringUtils.equals("X",nckdTiaojiatype)) {
                desc.append("价格调减");
            }
            BigDecimal nckdTiaozjiage = entry.getBigDecimal("nckd_tiaojiafudu_t");
            desc.append(nckdTiaozjiage.setScale(2)).append("元。");
            entry.set("nckd_desc_t",desc+"。");
        }
    }

    private void joinPtDesc(DynamicObject bill, List<DynamicObject> dai, List<DynamicObject> san) {
        DynamicObjectCollection nckdPutentry = bill.getDynamicObjectCollection("nckd_putentry");
        for (DynamicObject entry : nckdPutentry) {
            StringBuilder desc= new StringBuilder();

            DynamicObject nckdQuyus = entry.getDynamicObject("nckd_quyus");
            if(null != nckdQuyus){
                desc.append(nckdQuyus.getString("name")).append("区域");
            }
            DynamicObjectCollection nckdXqys = entry.getDynamicObjectCollection("nckd_xqys");
            if(!nckdXqys.isEmpty()){
                List<String> xqyNames = nckdXqys.stream().map(x -> x.getDynamicObject("fbasedataid").getString("name")).distinct().collect(Collectors.toList());
                if (xqyNames.size()==1){
                    desc.append("，").append(xqyNames.get(0)).append("小区域");
                }else{
                    desc.append("，").append(String.join("、", xqyNames)).append("等小区域");
                }
            }
            DynamicObjectCollection nckdSaletypes = entry.getDynamicObjectCollection("nckd_saletypes");
            if(!nckdSaletypes.isEmpty()){
                List<String> saletypeNames = nckdSaletypes.stream().map(x -> x.getDynamicObject("fbasedataid").getString("name")).distinct().collect(Collectors.toList());
                if (saletypeNames.size()==1){
                    desc.append("，").append(saletypeNames.get(0)).append("销售类型");
                }else{
                    desc.append("，").append(String.join("、", saletypeNames)).append("等销售类型");
                }
            }
            DynamicObjectCollection nckdTjjd = entry.getDynamicObjectCollection("nckd_tjjd");
            if(!nckdTjjd.isEmpty()){
                List<String> tjjdNames = nckdTjjd.stream().map(x -> x.getDynamicObject("fbasedataid").getString("name")).distinct().collect(Collectors.toList());
                if (tjjdNames.size()==1){
                    desc.append("，").append(tjjdNames.get(0)).append("调价基地");
                }else{
                    desc.append("，").append(String.join("、", tjjdNames)).append("等调价基地");
                }
            }
            Date nckdTzsxsj = entry.getDate("nckd_tzsxsj");
            String format = sdf.format(nckdTzsxsj);
            desc.append("自").append(format).append("起");

            DynamicObjectCollection nckdTjdetailentry = entry.getDynamicObjectCollection("nckd_tjdetailentry");

            List<String> daicount=new ArrayList<String>();
            List<String> sancount=new ArrayList<String>();
            for (DynamicObject tjdetailentry : nckdTjdetailentry) {
                DynamicObject nckdMaterialhang = tjdetailentry.getDynamicObject("nckd_materialhang");
                String name = nckdMaterialhang.getString("name");
                if (name.contains("袋装")){
                    daicount.add(name);
                }else if (name.contains("散装")){
                    sancount.add(name);
                }
            }

            String materialStr="对";
            if (!daicount.isEmpty()){
                if (daicount.size()== dai.size()){
                    materialStr=materialStr+"全部袋装";
                }else {
                    materialStr=  materialStr+String.join("、",daicount);
                }
            }

            if (!sancount.isEmpty()){
                if (sancount.size()== san.size()){
                    materialStr=materialStr+",全部散装";
                }else {
                    materialStr=  materialStr+String.join("、",sancount);
                }
            }

            desc.append(materialStr).append("等物料");

            String nckdTiaojiatype = entry.getString("nckd_tiaojiatype");
            if (StringUtils.equals("S",nckdTiaojiatype)){
                desc.append("价格调增");
            } else if (StringUtils.equals("X",nckdTiaojiatype)) {
                desc.append("价格调减");
            }
            BigDecimal nckdTiaozjiage = entry.getBigDecimal("nckd_tiaozjiage");
            desc.append(nckdTiaozjiage.setScale(2)).append("元。");
            StringBuilder lwsr=new StringBuilder();
            DynamicObjectCollection nckdSubentryentity = entry.getDynamicObjectCollection("nckd_subentryentity");
            for (DynamicObject subentryItem : nckdSubentryentity) {
                DynamicObject xQuyus = subentryItem.getDynamicObject("nckd_xqys_d");
                if(null != xQuyus){
                    lwsr.append(xQuyus.getString("name")).append("小区域");
                }

                DynamicObject saletype = subentryItem.getDynamicObject("nckd_saletypes_d");
                if(null != saletype){
                    lwsr.append(saletype.getString("name")).append("销售类型");
                }

                DynamicObject customer = subentryItem.getDynamicObject("nckd_customer");
                if(null != customer){
                    lwsr.append(customer.getString("name")).append("客户");
                }
                lwsr.append("不调整");
            }
            desc.append(lwsr);
            entry.set("nckd_desc",desc.toString()+"。");
        }
    }

    @Override
    public void beforeExecuteOperationTransaction(BeforeOperationArgs e) {
        super.beforeExecuteOperationTransaction(e);
    }

    @Override
    public void endOperationTransaction(EndOperationTransactionArgs e) {
        super.endOperationTransaction(e);
    }

    @Override
    public void afterExecuteOperationTransaction(AfterOperationArgs e) {
        super.afterExecuteOperationTransaction(e);
    }
}